"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _commodoFieldsObject = require("commodo-fields-object");

var _fields = require("@commodo/fields");

var _validation = require("@webiny/validation");

var _dbArgs = _interopRequireDefault(require("./dbArgs"));

var _paginateBatch = require("./paginateBatch");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const CreateDataModel = (0, _fields.withFields)({
  tenant: (0, _fields.string)({
    validation: _validation.validation.create("required")
  }),
  name: (0, _fields.string)({
    validation: _validation.validation.create("required,minLength:3")
  }),
  slug: (0, _fields.string)({
    validation: _validation.validation.create("required,minLength:3")
  }),
  description: (0, _fields.string)({
    validation: _validation.validation.create("maxLength:500")
  }),
  permissions: (0, _commodoFieldsObject.object)({
    list: true,
    validation: _validation.validation.create("required")
  })
})();
const UpdateDataModel = (0, _fields.withFields)({
  name: (0, _fields.string)({
    validation: _validation.validation.create("minLength:3")
  }),
  description: (0, _fields.string)({
    validation: _validation.validation.create("maxLength:500")
  }),
  permissions: (0, _commodoFieldsObject.object)({
    list: true
  })
})();

var _default = context => {
  const {
    db
  } = context;
  return {
    async getGroup(tenant, slug) {
      const [[group]] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: `G#${slug}`
        },
        limit: 1
      }));
      return group;
    },

    async listGroups(tenant) {
      const [groups] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: {
            $beginsWith: "G#"
          }
        }
      }));
      return groups;
    },

    async createGroup(tenant, data) {
      const identity = context.security.getIdentity();

      if (await this.getGroup(tenant, data.slug)) {
        throw {
          message: `Group with slug "${data.slug}" already exists.`,
          code: "GROUP_EXISTS"
        };
      }

      await new CreateDataModel().populate(_objectSpread(_objectSpread({}, data), {}, {
        tenant: tenant.id
      })).validate();

      const group = _objectSpread(_objectSpread({
        tenant: tenant.id,
        system: false
      }, data), {}, {
        createdOn: new Date().toISOString(),
        createdBy: identity ? {
          id: identity.id,
          displayName: identity.displayName,
          type: identity.type
        } : null
      });

      await db.create(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        data: _objectSpread({
          PK: `T#${tenant.id}`,
          SK: `G#${data.slug}`,
          TYPE: "security.group"
        }, group)
      }));
      return group;
    },

    async updateGroup(tenant, slug, data) {
      const model = await new UpdateDataModel().populate(data);
      await model.validate();
      await db.update(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: `G#${slug}`
        },
        data: await model.toJSON({
          onlyDirty: true
        })
      }));
      return true;
    },

    async deleteGroup(tenant, slug) {
      await db.delete(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: `G#${slug}`
        }
      }));
      return true;
    },

    async updateUserLinks(tenant, group) {
      const [links] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          GSI1_PK: `T#${tenant.id}`,
          GSI1_SK: {
            $beginsWith: `G#${group.slug}#`
          }
        }
      }));
      await (0, _paginateBatch.paginateBatch)(links, 25, async links => {
        const batch = db.batch();

        for (let i = 0; i < links.length; i++) {
          batch.update(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
            query: {
              PK: links[i].PK,
              SK: links[i].SK
            },
            data: Object.assign(links[i], {
              group: {
                slug: group.slug,
                name: group.name,
                permissions: group.permissions
              }
            })
          }));
        }

        await batch.execute();
      });
    }

  };
};

exports.default = _default;
//# sourceMappingURL=groups.crud.js.map