"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Users = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _mdbid = _interopRequireDefault(require("mdbid"));

var _lodash = _interopRequireDefault(require("lodash.clonedeep"));

var _fields = require("@commodo/fields");

var _validation = require("@webiny/validation");

var _apiSecurity = require("@webiny/api-security");

var _handlerGraphql = require("@webiny/handler-graphql");

var _error = _interopRequireDefault(require("@webiny/error"));

var _base = require("./base.crud");

var _UserPlugin = require("../plugins/UserPlugin");

var _dbArgs = _interopRequireDefault(require("./dbArgs"));

var _users = require("./users.loaders");

var _users2 = _interopRequireDefault(require("./users.validation"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const CreateAccessTokenDataModel = (0, _fields.withFields)({
  name: (0, _fields.string)({
    validation: _validation.validation.create("required,maxLength:100")
  }),
  token: (0, _fields.string)({
    validation: _validation.validation.create("required,maxLength:64")
  })
})();
const UpdateAccessTokenDataModel = (0, _fields.withFields)({
  name: (0, _fields.string)({
    validation: _validation.validation.create("required")
  })
})();

class Users extends _base.Base {
  constructor(context) {
    super(context);
    (0, _defineProperty2.default)(this, "loaders", void 0);
    context.plugins.register(_users2.default);
    this.loaders = new _users.UserLoaders(context);
  }

  get plugins() {
    return this.context.plugins.byType(_UserPlugin.UserPlugin.type);
  }

  async login() {
    const {
      security
    } = this.context;
    const identity = security.getIdentity();

    if (!identity) {
      throw new _apiSecurity.NotAuthorizedError();
    }

    let user = await this.getUser(identity.id, {
      auth: false
    });
    let firstLogin = false;

    if (!user) {
      firstLogin = true; // Create a "Security User"

      user = await this.createUser({
        login: identity.id,
        firstName: identity.firstName || "",
        lastName: identity.lastName || "",
        avatar: identity.avatar
      }, {
        auth: false
      });
    }

    await this.executeCallback("onLogin", {
      context: this.context,
      user,
      firstLogin
    });
    return user;
  }

  async createToken(login, data) {
    const {
      name,
      token
    } = data;
    await new CreateAccessTokenDataModel().populate({
      name,
      token
    }).validate();

    const tokenData = _objectSpread(_objectSpread({}, data), {}, {
      id: (0, _mdbid.default)(),
      name,
      token,
      login,
      createdOn: new Date().toISOString()
    });

    await this.context.db.create(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      data: _objectSpread({
        PK: `U#${login}`,
        SK: `PAT#${tokenData.id}`,
        GSI1_PK: `PAT`,
        GSI1_SK: token,
        TYPE: "security.pat"
      }, tokenData)
    }));
    return tokenData;
  }

  async createUser(data, options = {}) {
    const {
      db,
      security,
      tenancy
    } = this.context;
    const auth = "auth" in options ? options.auth : true;

    if (auth) {
      const permission = await security.getPermission("security.user");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }
    }

    const login = data.login.toLowerCase();
    const identity = security.getIdentity();

    if (await this.getUser(login, {
      auth: false
    })) {
      throw {
        message: "User with that login already exists.",
        code: "USER_EXISTS"
      };
    }

    const user = _objectSpread(_objectSpread({}, data), {}, {
      login,
      createdOn: new Date().toISOString(),
      createdBy: identity ? {
        id: identity.id,
        displayName: identity.displayName,
        type: identity.type
      } : null
    });

    await this.executeCallback("beforeCreate", {
      context: this.context,
      user,
      inputData: data
    });
    await db.create(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      data: _objectSpread({
        PK: `U#${user.login}`,
        SK: "A",
        TYPE: "security.user"
      }, user)
    }));
    await this.executeCallback("afterCreate", {
      context: this.context,
      user,
      inputData: data
    });
    const tenant = tenancy.getCurrentTenant();
    const group = await security.groups.getGroup(tenant, data.group);
    await security.users.linkUserToTenant(user.login, tenant, group);
    return user;
  }

  async deleteToken(login, tokenId) {
    await this.context.db.delete(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: `PAT#${tokenId}`
      }
    }));
    return true;
  }

  async deleteUser(login) {
    const {
      db,
      security
    } = this.context;
    const permission = await security.getPermission("security.user");

    if (!permission) {
      throw new _apiSecurity.NotAuthorizedError();
    }

    const user = await this.getUser(login);
    const identity = security.getIdentity();

    if (!user) {
      throw new _handlerGraphql.NotFoundError(`User "${login}" was not found!`);
    }

    if (user.login === identity.id) {
      throw new _error.default(`You can't delete your own user account.`);
    }

    await this.executeCallback("beforeDelete", {
      context: this.context,
      user
    });
    const [items] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: {
          $gt: " "
        }
      }
    }));
    const batch = db.batch();

    for (let i = 0; i < items.length; i++) {
      batch.delete(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: items[i].PK,
          SK: items[i].SK
        }
      }));
    }

    await batch.execute();
    this.loaders.clearLoadersCache(login);
    await this.executeCallback("afterDelete", {
      context: this.context,
      user
    });
    return true;
  }

  async getPersonalAccessToken(login, tokenId) {
    const [[token]] = await this.context.db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: `PAT#${tokenId}`
      }
    }));
    return token;
  }

  async getUser(login, options = {}) {
    const {
      security
    } = this.context;
    const auth = "auth" in options ? options.auth : true;

    if (auth) {
      const permission = await security.getPermission("security.user");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }
    }

    return this.loaders.getUser.load(login);
  }

  async getUserAccess(login) {
    return this.loaders.getUserAccess.load(login);
  }

  async getUserByPersonalAccessToken(token) {
    const [[pat]] = await this.context.db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        GSI1_PK: "PAT",
        GSI1_SK: token
      }
    }));

    if (!pat) {
      return null;
    }

    return await this.getUser(pat.login, {
      auth: false
    });
  }

  async linkUserToTenant(login, tenant, group) {
    const data = {
      tenant: {
        id: tenant.id,
        name: tenant.name
      },
      group: {
        slug: group.slug,
        name: group.name,
        permissions: group.permissions
      }
    };
    await this.context.db.create(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      data: _objectSpread({
        PK: `U#${login}`,
        SK: `LINK#T#${tenant.id}#G#${group.slug}`,
        GSI1_PK: `T#${tenant.id}`,
        GSI1_SK: `G#${group.slug}#U#${login}`,
        TYPE: "security.user2tenant"
      }, data)
    }));
    await this.loaders.addDataLoaderAccessCache(login, data);
  }

  async listTokens(login) {
    const [tokens] = await this.context.db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: {
          $beginsWith: "PAT#"
        }
      }
    }));
    return tokens;
  }

  async listUsers(params = {}) {
    const {
      db,
      security,
      tenancy
    } = this.context;
    const auth = "auth" in params ? params.auth : true;

    if (auth) {
      const permission = await security.getPermission("security.user");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }
    }

    const tenant = params.tenant || tenancy.getCurrentTenant().id;
    const [users] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        GSI1_PK: `T#${tenant}`,
        GSI1_SK: {
          $beginsWith: "G#"
        }
      }
    }));
    const batch = db.batch();

    for (let i = 0; i < users.length; i++) {
      batch.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: users[i].PK,
          SK: "A"
        }
      }));
    }

    const results = await batch.execute();
    return results.map(res => res[0][0]);
  }

  async unlinkUserFromTenant(login, tenant) {
    const {
      db
    } = this.context;
    const [[link]] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: {
          $beginsWith: `LINK#T#${tenant.id}#G#`
        }
      },
      limit: 1
    }));
    await db.delete(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: link.PK,
        SK: link.SK
      }
    }));
    await this.loaders.deleteDataLoaderAccessCache(login, tenant);
  }

  async updateToken(login, tokenId, data) {
    const {
      name
    } = data;
    await new UpdateAccessTokenDataModel().populate({
      name
    }).validate();
    await this.context.db.update(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: `PAT#${tokenId}`
      },
      data: {
        name
      }
    }));
    return {
      name
    };
  }

  async updateUser(login, data) {
    const {
      security,
      tenancy,
      db
    } = this.context;
    const permission = await security.getPermission("security.user");

    if (!permission) {
      throw new _apiSecurity.NotAuthorizedError();
    }

    const user = await security.users.getUser(login);

    if (!user) {
      throw new _handlerGraphql.NotFoundError(`User "${login}" was not found!`);
    }

    const updateData = (0, _lodash.default)(data);
    await this.executeCallback("beforeUpdate", {
      context: this.context,
      user,
      updateData,
      inputData: data
    });
    Object.assign(user, updateData);
    await db.update(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
      query: {
        PK: `U#${login}`,
        SK: "A"
      },
      data: user
    }));
    await this.executeCallback("afterUpdate", {
      context: this.context,
      user,
      inputData: data
    });
    await this.loaders.updateDataLoaderUserCache(login, updateData);
    this.loaders.clearDataLoaderAccessCache(login);

    if (data.group) {
      const tenant = tenancy.getCurrentTenant();
      await security.users.unlinkUserFromTenant(user.login, tenant);
      const group = await security.groups.getGroup(tenant, data.group);
      await security.users.linkUserToTenant(user.login, tenant, group);
    }

    return user;
  }

}

exports.Users = Users;
//# sourceMappingURL=users.crud.js.map